//
//  IsingStats.m
//  IsingCocoa
//
//  Created by Cameron Hayne on 14/03/05.
//  Copyright Hayne of Tintagel 2005
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#import "IsingStats.h"
#import "IsingModel.h"
#import "IsingSpinView.h"

static NSString *fpsToString(float fps)
{
    if (fps <= 1.5)
    {
        return [NSString stringWithFormat: @"%.1f", fps];
    }
    else
    {
        return [NSString stringWithFormat: @"%.0f", fps];
    }
}

static NSString *mspsToString(float msps)
{
    if (msps <= 2.0)
    {
        return [NSString stringWithFormat: @"%.2f", msps];
    }
    else
    {
        return [NSString stringWithFormat: @"%.1f", msps];
    }
}

@implementation IsingWindowCtl (IsingStats)

- (void) updateStatistics
{
    NSTimeInterval minTimeBetweenUpdates = 0.5;
    NSTimeInterval now = [NSDate timeIntervalSinceReferenceDate];
    NSTimeInterval elapsedSec = now - info.lastStatsUpdateTime;
    if (elapsedSec < minTimeBetweenUpdates)
    {
        // no need to update statistics more frequently than a few times per second
        return;
    }
    
    info.elapsedSecSinceReset += elapsedSec;
    
    int totalModelUpdates = [model numSweeps];
    int totalViewUpdates  = [spinView numRedraws];
    int modelUpdates = totalModelUpdates - info.lastModelUpdates;
    int viewUpdates = totalViewUpdates - info.lastViewUpdates;

    info.numModelUpdatesSinceReset += modelUpdates;
    info.numViewUpdatesSinceReset += viewUpdates;
    
    float modelFps     = modelUpdates / elapsedSec;
    float modelAvgFps  = info.numModelUpdatesSinceReset / info.elapsedSecSinceReset;
    float modelMsps    = modelFps * info.numMegaPoints;
    float modelAvgMsps = modelAvgFps * info.numMegaPoints;
    
    float viewFps      = viewUpdates / elapsedSec;
    float viewAvgFps   = info.numViewUpdatesSinceReset / info.elapsedSecSinceReset;
    float viewMsps     = viewFps * info.numMegaPoints;
    float viewAvgMsps  = viewAvgFps * info.numMegaPoints;
    
    [modelFpsLabel     setStringValue: fpsToString(modelFps)];
    [modelAvgFpsLabel  setStringValue: fpsToString(modelAvgFps)];
    [modelMspsLabel    setStringValue: mspsToString(modelMsps)];
    [modelAvgMspsLabel setStringValue: mspsToString(modelAvgMsps)];
    
    [viewFpsLabel      setStringValue: fpsToString(viewFps)];
    [viewAvgFpsLabel   setStringValue: fpsToString(viewAvgFps)];
    [viewMspsLabel     setStringValue: mspsToString(viewMsps)];
    [viewAvgMspsLabel  setStringValue: mspsToString(viewAvgMsps)];
    
    info.lastStatsUpdateTime = now;
    info.lastModelUpdates = totalModelUpdates;
    info.lastViewUpdates = totalViewUpdates;
    
    [magnetizationLabel setStringValue:
        [NSString stringWithFormat: @"%.2f", [model magnetization]]];
    [nn_correlationLabel setStringValue:
        [NSString stringWithFormat: @"%.2f", [model nnCorrelation]]];
}

- (void)resetFpsBookkeeping
{
    info.elapsedSecSinceReset = 0.0;
    info.numModelUpdatesSinceReset = 0;
    info.numViewUpdatesSinceReset = 0;
}

- (void)resetStats
{
    [self resetFpsBookkeeping];
    [self resetStatsUpdateTime];
    IsingSize mSize = [model size];
    info.numMegaPoints = mSize.width * mSize.height / 1.0e6;
    info.lastModelUpdates = 0;
    info.lastViewUpdates = 0;
}

- (void)resetStatsUpdateTime
{
    NSTimeInterval now = [NSDate timeIntervalSinceReferenceDate];
    info.lastStatsUpdateTime = now;
}

@end
