//
//  IsingWindowCtl.h
//  IsingCocoa
//
//  Created by Cameron Hayne on Tue May 21 2002.
//  Copyright Hayne of Tintagel 2005
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#import <Cocoa/Cocoa.h>
#import "ising.h" // for IsingSize
@class IsingSpinView;
@class IsingViewCtl;
@class IsingModel;
@protocol IsingView;

typedef struct
{
    NSTimeInterval      startTime;
    NSTimeInterval	elapsedSecSinceReset;
    NSTimeInterval	lastStatsUpdateTime;
    int			numModelUpdatesSinceReset;
    int			numViewUpdatesSinceReset;
    int			lastModelUpdates;
    int			lastViewUpdates;
    float 		numMegaPoints;
} UpdateInfo;


@interface IsingWindowCtl : NSWindowController
{    
    // sliders
    IBOutlet id              temperatureControl; // enclosure of slider & textfield
    IBOutlet id              fieldControl;       // enclosure of slider & textfield
    
    // toolbar items
    IBOutlet id              updateRateSlider;
    NSToolbarItem           *pauseToolbarItem;
    NSToolbarItem           *fieldToolbarItem;
    
    // statistics drawer
    IBOutlet id              statsDrawer;
    IBOutlet id              modelFpsLabel;
    IBOutlet id              modelAvgFpsLabel;
    IBOutlet id              modelMspsLabel;
    IBOutlet id              modelAvgMspsLabel;
    IBOutlet id              viewFpsLabel;
    IBOutlet id              viewAvgFpsLabel;
    IBOutlet id              viewMspsLabel;
    IBOutlet id              viewAvgMspsLabel;
    IBOutlet id              magnetizationLabel;
    IBOutlet id              nn_correlationLabel;
    IBOutlet id              avgFpsResetButton;
    
    IBOutlet IsingSpinView  *spinView;
    IBOutlet IsingModel     *model;
    IsingViewCtl            *viewController;
    NSTimer                 *viewUpdateTimer;
    BOOL                     needsViewUpdate;
    int                      lastSweepDisplayed;
    UpdateInfo               info; // used for calculating update rates
}

- (IBAction)pauseButtonAction:(id)sender;
- (IBAction)toggleMagneticFieldAction:(id)sender;
- (IBAction)modelUpdateRateAction:(id)sender;
- (IBAction)statsDrawerAction:(id)sender;
- (IBAction)resetAvgFpsAction:(id)sender;
- (IBAction)setSizeAction:(id)sender;
- (IBAction)setSpinsAction:(id)sender;
- (IBAction)blockSpinsAction:(id)sender;

- (void)initModelSize:(IsingSize)size;
- (void)addIsingView:(id <IsingView>)aView;

- (IsingModel *)model;
- (IsingSpinView *)spinView;
- (void)setNeedsViewUpdate:(BOOL)flag;

@end
