//
//  SizeConfigPanelCtl.m
//  IsingCocoa
//
//  Created by Cameron Hayne on 08/05/05.
//  Copyright Hayne of Tintagel 2005
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#import "SizeConfigPanelCtl.h"
#import "IsingWindowCtl.h"
#import "IsingModel.h"
#import "IsingSpinView.h"
#import "WindowSizing.h"

@interface SizeConfigPanelCtl (Private)
- (void)setupRangesForScaleFactor:(float)scaleFactor;
@end

@implementation SizeConfigPanelCtl

- (id)initForWindowCtl:(IsingWindowCtl *)aWindowCtl
{
    self = [super initWithWindowNibName: @"SizeConfigPanel"];
    if (self == nil)
    {
        return nil;
    }

    //NSLog(@"SizeConfigPanelCtl.init");
    isingWindowCtl = aWindowCtl;
    mainWindow = [isingWindowCtl window];
    spinView = [isingWindowCtl spinView];
    model = [isingWindowCtl model];
    
    [self window]; // force loading of the NIB
    return self;
}

- (void)dealloc
{
    //NSLog(@"SizeConfigPanelCtl.dealloc");
    [super dealloc];
}

- (void)windowDidLoad
{
    //NSLog(@"SizeConfigPanelCtl.windowDidLoad");
    IsingSize modelSize = [model size];
    NSSize viewSize = [spinView frame].size;
    float scaleFactor = modelSize.width / viewSize.width;
    
    [widthTxf  setIntValue: modelSize.width];
    [heightTxf setIntValue: modelSize.height];
    [scaleCmb setStringValue: [NSString stringWithFormat: @"%2.2f", scaleFactor]];
    [self setupRangesForScaleFactor: scaleFactor];

    [NSApp beginSheet: sheet
       modalForWindow: mainWindow
        modalDelegate: nil
       didEndSelector: NULL
          contextInfo: nil];
    
    // We need to retain the SizeConfigPanelCtl object since it is released
    // in IsingWindowCtl and we need it to stick around until the sheet is closed
    [self retain];
}

- (void)setupRangesForScaleFactor:(float)scaleFactor
{    
    // want the view frame to be at least 150 pixels square
    int minPixels = 150;
    NSSize maxViewSize = [mainWindow maxUsableViewSize: spinView];
    int minWidth  = (int)roundf(minPixels * scaleFactor);
    int maxWidth  = (int)(maxViewSize.width * scaleFactor);
    int minHeight = (int)roundf(minPixels * scaleFactor);
    int maxHeight = (int)(maxViewSize.height * scaleFactor);
    [widthRangeLbl setStringValue:
        [NSString stringWithFormat: @"(min %d,\n max %d\n rounded to nearest %d)",
                                       minWidth, maxWidth, WORD_SIZE]];
    [heightRangeLbl setStringValue:
        [NSString stringWithFormat: @"(min %d,\n max %d)",
                                       minHeight, maxHeight]];
    int width  = [widthTxf intValue];
    int height = [heightTxf intValue];
    if (width < minWidth)
    {
        [widthTxf setIntValue: minWidth];
    }
    if (width > maxWidth)
    {
        [widthTxf setIntValue: maxWidth];
    }
    if (height < minHeight)
    {
        [heightTxf setIntValue: minHeight];
    }
    if (height > maxHeight)
    {
        [heightTxf setIntValue: maxHeight];
    }
}

- (IBAction)scaleCmbAction:(id)sender
{
    float scaleFactor = [sender floatValue];

    [self setupRangesForScaleFactor: scaleFactor];
}

- (IBAction)closeSheet:(id)sender
{
    float minScaleFactor = 0.01;
    float maxScaleFactor = 100.0;
    int minPixels = 150;

    [sheet orderOut: nil];
    [NSApp endSheet: sheet];
    
    // We no longer need the SizeConfigPanelCtl object
    [self autorelease];
    
    if (sender != okBtn)
    {
        // user cancelled
        return;
    }
    
    float scaleFactor = [scaleCmb floatValue];
    int width  = [widthTxf intValue];
    int height = [heightTxf intValue];
    
    if (scaleFactor < minScaleFactor)
    {
        scaleFactor = minScaleFactor;
    }
    if (scaleFactor > maxScaleFactor)
    {
        scaleFactor = maxScaleFactor;
    }
    
    // want the view frame to be at least 'minPixels' pixels square
    NSSize maxViewSize = [mainWindow maxUsableViewSize: spinView];
    int minWidth  = (int)roundf(minPixels * scaleFactor);
    int maxWidth  = (int)(maxViewSize.width * scaleFactor);
    int minHeight = (int)roundf(minPixels * scaleFactor);
    int maxHeight = (int)(maxViewSize.height * scaleFactor);
    if (width < minWidth)
    {
        width = minWidth;
    }
    if (width > maxWidth)
    {
        width = maxWidth;
    }
    if (height < minHeight)
    {
        height = minHeight;
    }
    if (height > maxHeight)
    {
        height = maxHeight;
    }
    IsingSize proposedSize = makeIsingSize(width, height);
    
    // ask the model to suggest an acceptable size
    IsingSize acceptableSize = [model acceptableSize: proposedSize];
    [isingWindowCtl initModelSize: acceptableSize];
    
    float scaledViewWidth  = acceptableSize.width / scaleFactor;
    float scaledViewHeight = acceptableSize.height / scaleFactor;
    //NSLog(@"scaledViewWidth = %f scaledViewHeight = %f", scaledViewWidth, scaledViewHeight);

    // determine how much we need to resize the window
    NSSize viewSize = [spinView frame].size;
    int deltaWidth  = (int)roundf(scaledViewWidth - viewSize.width);
    int deltaHeight = (int)roundf(scaledViewHeight - viewSize.height);
    [mainWindow changeWidthBy: deltaWidth
                     heightBy: deltaHeight
                      animate: YES];
}

@end
