//
//  Util.m
//  IsingCocoa
//
//  Created by Cameron Hayne on 07/11/04.
//  Copyright Hayne of Tintagel 2005
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#import "Util.h"

@implementation Util

+ (void) checkAppKitVersion
{
    // we require Panther or greater due to the use of NSView.setHidden
    
    if (floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_2)
    {
        NSString *title = @"Not supported";
        NSString *msg = @"This program requires Panther (OS 10.3) or later";
        NSRunAlertPanel(title, msg, nil, nil, nil);
        [NSApp terminate: nil];
    }
}

// The following is a utility routine that is used for postponing a selector
// until after some activity is finished.
// It is designed to be repeatedly called while the activity is ongoing
// and it will result in the selector being invoked exactly once
// after the activity has finished.
+ (void)performOnce:(SEL)aSelector
         withTarget:(id)aTarget
         withObject:(id)anArgument
         afterDelay:(NSTimeInterval)delay
{
    // cancel any pending requests of this type
    [NSObject cancelPreviousPerformRequestsWithTarget: aTarget
                                             selector: aSelector
                                               object: anArgument];
    // schedule the request
    [aTarget performSelector: aSelector
                  withObject: anArgument
                  afterDelay: delay];
}

// maximum size a window on the main screen can be
// without overlapping the menubar or the dock
+ (NSSize)maxUsableWindowSize
{
    return [[NSScreen mainScreen] visibleFrame].size;
}

@end
